<?php 

namespace TekIntegra\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Integracao {
    
    protected $connection;
    protected $novoCodigo;
    protected $version;
    
    public function __construct($connection,$novoCodigo) {
        $this->connection = $connection;
        $this->novoCodigo = $novoCodigo;
        $this->version = json_decode(file_get_contents(__DIR__ . '/../../../../mobile/bower.json'), true)['version'];
    }
	
	//Rota de Envio
	public function realizaIntegracao($request,$response) {	       
	   try {	 
			$this->connection->beginTransaction();
			
			$info = json_encode($request->getParameters());
			$dados = json_decode($info, true);
			/* if (is_array($dados) === false) {
				throw new \Exception('Dados não foram enviados corretamente. ('. $info .')');
			}*/
            
			$nrorg = $dados['NRORGINT'];

		    $params = json_decode(file_get_contents('php://input'), true);
			
			$this->validaOrganizacao($nrorg);
			
    	    
    	    $this->recebeDados($dados, $nrorg);    	    
            $this->atualizaProdutoEmReceita($nrorg);
            
            $remessa = $this->retornaNrRemessa(true);
            
            $this->criaNovaRemessa($remessa,$nrorg); 
            
            $retorno = 
            [
                "NRREMESSA" => $remessa,
                "NRORGINT" => $nrorg,
                "QTPRODREC" => $dados['QTPRODREC'],
                "PRODUTOS" => []
            ]; 
            
            // Lista até mil produtos   
            $produtos = $this->retornaDadosProduto();
            $retorno['PRODUTOS'] = $produtos;
            $retorno['QTPRODRET'] = count($produtos);    
            
            if ($retorno['QTPRODRET'] > 0) {
                foreach ($produtos as $produto) {
                    $this->insertLogProd($remessa,$produto['CDPRODUTO'], $nrorg);
                }
            }
            
			$json = json_encode($retorno);
			echo $json;
            $this->connection->commit();
			
    	}catch (\Exception $e) {
    	    //var_dump($this->connection->isConnected());
    	    if($this->connection->isConnected()){
    	        $this->connection->rollback();
    	    }
   			echo json_encode($e->getMessage() . ' Versão: '.$this->version);
        }
	}
	
	// Rota de reenvio
	public function reenvioRemessa($request,$response) {
	    try {
			$this->connection->beginTransaction();
    	    $info = json_encode($request->getParameters());
			$dados = json_decode($info, true);
			/*if (is_array($dados) === false) {
				throw new \Exception('Dados não foram enviados corretamente. ('. $info .')');
			}*/
			
			$nrorg = $dados['NRORGINT'];
			
			$this->validaOrganizacao($nrorg);
						
            $this->recebeDados($dados, $nrorg);
            
            // PEGA A MAIOR REMESSA
            $remessa = $this->retornaNrRemessa(false);
            
            $retorno = 
                [
                    "NRREMESSA" => $remessa,
                    "NRORGINT" => $nrorg,
                    "QTPRODREC" => $dados['QTPRODREC'],
                    "PRODUTOS" => []
                ]; 
                
            // Lista até mil produtos   
            $produtos = $this->retornaDadosJaEnviados($remessa, $nrorg);
            $retorno['PRODUTOS'] = $produtos;
            $retorno['QTPRODRET'] = count($produtos);
            
            $this->atualizaDataEnvio($remessa);
               
			$json = json_encode($retorno);
			echo $json;
            $this->connection->commit();
               
	    }catch (\Exception $e) {
    	    //var_dump($this->connection->isConnected());
    	    if($this->connection->isConnected()){
    	        $this->connection->rollback();
    	    }
			echo json_encode($e->getMessage() . ' Versão: '.$this->version);
        }
	}
	
	//Rota de preparação de ambiente 
	public function preparaAmbiente($request,$response){
	    try {
	        $this->connection->beginTransaction();
            
            $info = json_encode($request->getParameters());
			$dados = json_decode($info, true);
			
           /* if (is_array($dados) === false) {
				throw new \Exception('Dados não foram enviados corretamente. ('. $info .')');
			}*/
            
			$nrorg = $dados['NRORGINT'];

            $this->validaOrganizacao($nrorg);
			if($this->primeiraSincronizacao($nrorg)){
			    
			    $tabelas = ['LIMNUTRI', 'DIAGNUTR', 'TRABNUTRIREC', 'PRINATIVNUTR', 'NUTRNECE', 'NUTRECOMPACI', 'ITEQUIVALE',
			                'CARDPADR', 'CARDCONSNUTR','NUTRPROD','NUTRECOM','GREQUIVALE','NUTRIENTE', 'GRUPNUTR'];

			    $this->backupTabelasNutr($tabelas);
			    $this->excluiTabelasNutr($dados, $tabelas);
			    
			    foreach($dados['INFONUTR'] as $infoNutr){
			        $this->verificaGrupNutr($infoNutr['CDGRUPNUTR'],$infoNutr['NMGRUPNUTR']);

					$this->verificaUnidade($infoNutr['SGUNIDADE'], $infoNutr['DSUNIDADE']);

					$this->verificaNutriente($infoNutr);
			    }
			    
			    $this->updateParams($dados);
			    
			    $remessa = $this->retornaNrRemessa(true);
                $remessa = $this->criaNovaRemessa($remessa,$nrorg);
                
                $retorno =["MENSAGEM" => "PREPARAÇÃO DE AMBIENTE"]; 
			    
			    $json = json_encode($retorno);
			    
                
			}else $json = json_encode(["MENSAGEM"  => $nrorg."-CLIENTE ANTIGO"]);  
			 
            echo $json;
            $this->connection->commit();
	    }catch (\Exception $e) {
    	    if($this->connection->isConnected()){
    	        $this->connection->rollback();
    	    }
    	    echo json_encode($e->getMessage() . ' Versão: '.$this->version);
   			
        }finally{
            $this->configNovaVersao();
        }
	}
	
	private function backupTabelasNutr($tabelas){
        try{
            
            foreach($tabelas as $tabela){
                $query = "CREATE TABLE ".$tabela."_BKPAUTOTEKNUTRI AS SELECT * FROM ".$tabela;
                $stmt = $this->connection->prepare($query);
                $stmt->execute();
            }
            
            //backup PARAMGERAL
            $query = "CREATE TABLE PARAMGERAL_BKPAUTOTEKNUTRI AS SELECT CDPARGERAL, CDPROTEINA,
                      CDLIPIDIOS, CDCARBOHIDRA, CDALCOOL, CDPROTANIMAL, CDPROTVEGETA,
                      CDACUCARES, CDACIDGRAXOS, CDCALORIAS FROM PARAMGERAL";
            $stmt = $this->connection->prepare($query);
            $stmt->execute();
            
            
            //backup PARAMPLANEJA
            $query = "CREATE TABLE PARAMPLANEJA_BKPAUTOTEKNUTRI AS SELECT CDNUTRRELAT1, CDNUTRRELAT2, CDNUTRRELAT3, CDNUTRRELAT4 FROM PARAMPLANEJA";
            $stmt = $this->connection->prepare($query);
            $stmt->execute();
            
            
            //updates
            $query = "UPDATE PARAMGERAL SET CDPROTEINA = NULL, CDLIPIDIOS = NULL,
                                            CDCARBOHIDRA = NULL,CDALCOOL = NULL,
                                            CDPROTANIMAL = NULL, CDPROTVEGETA = NULL,
                                            CDACUCARES = NULL, CDACIDGRAXOS = NULL,
                                            CDCALORIAS = NULL, DTULTATU = SYSDATE";
            $stmt = $this->connection->prepare($query);
            $stmt->execute();
            
            $query = "UPDATE PARAMPLANEJA SET CDNUTRRELAT1 = NULL, CDNUTRRELAT2 = NULL, CDNUTRRELAT3 = NULL, CDNUTRRELAT4 = NULL, DTULTATU = SYSDATE";
            $stmt = $this->connection->prepare($query);
            $stmt->execute();
        }catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
	private function excluiTabelasNutr($dados, $tabelas){
        try{
            foreach($tabelas as $tabela){
                $query = "DELETE FROM ".$tabela;
                $stmt = $this->connection->prepare($query);
                $stmt->execute();
            }
            
            try{
                $query = "DROP TRIGGER TD_GREQUIVALE";
                $stmt = $this->connection->prepare($query);
                $stmt->execute();
                
                $query = "DROP TRIGGER TD_NUTRIENTE";
                $stmt = $this->connection->prepare($query);
                $stmt->execute();
            }catch (\Exception $e) {}
            
        }catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
        
    }
    
    private function updateParams ($dados){
        try{
            $paramgeral = $dados['PARAMGERAL'];
    
            //PARAMGERAL
            $query = "UPDATE PARAMGERAL SET CDPROTEINA =:CDPROTEINA, CDLIPIDIOS =:CDLIPIDIOS,
                                            CDCARBOHIDRA =:CDCARBOHIDRA,CDALCOOL =:CDALCOOL,
                                            CDPROTANIMAL =:CDPROTANIMAL, CDPROTVEGETA =:CDPROTVEGETA,
                                            CDACUCARES =:CDACUCARES, CDACIDGRAXOS =:CDACIDGRAXOS,
                                            CDCALORIAS =:CDCALORIAS, DTULTATU = SYSDATE";
            $params = array(
                  'CDPROTEINA'  => $paramgeral['CDPROTEINA'],
                  'CDLIPIDIOS'  => $paramgeral['CDLIPIDIOS'],
                  'CDCARBOHIDRA'=> $paramgeral['CDCARBOHIDRA'],
                  'CDALCOOL'    => $paramgeral['CDALCOOL'],
                  'CDPROTANIMAL'=> $paramgeral['CDPROTANIMAL'],
                  'CDPROTVEGETA'=> $paramgeral['CDPROTVEGETA'],
                  'CDACUCARES'  => $paramgeral['CDACUCARES'],
                  'CDACIDGRAXOS'=> $paramgeral['CDACIDGRAXOS'],
                  'CDCALORIAS'  => $paramgeral['CDCALORIAS']
            );
            
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            
        }catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

	private function primeiraSincronizacao($nrorgint){
	    $result = true;
	    
	    $query = "SELECT * FROM REMPRODNUTR
                  WHERE NRORGINT IS NULL OR NRORGINT = :NRORGINT
	              ORDER BY NRREMESSA ASC";
    	    
        $stmt = $this->connection->prepare($query);
        $params = array(
            "NRORGINT" =>  $nrorgint
        );
        $stmt->execute($params);
        $remessas = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(count($remessas) > 0) $result = false;
	    return $result;
	}
	
	private function recebeDados($dados, $nrorg) {
		if($this->verificaEnvioProdutos($dados)) { // Valida Recebimento
			if($dados['QTPRODRET'] === count($dados['PRODUTOS'])) {

				foreach ($dados['PRODUTOS'] as $produto) {
					//Apaga info nutricional
					$temNutriente = 'N';
					
					if($dados['IDCONSNUT'] === 'S') {  // Verifica se possui consultoria nutricional contratada 
						$this->deleteNutrProd($produto['CDPRODUTO']);
						  
						if($produto['IDATIVO'] === 'S') {
							$temNutriente = 'S';
							foreach ($produto[0] as $info) {
								$this->verificaGrupNutr($info['CDGRUPNUTR'],$info['NMGRUPNUTR']);

								$this->verificaUnidade($info['SGUNIDADE'], null);

								$this->verificaNutriente($info);

								$this->insertNutrProd($produto['CDPRODUTO'],$info);								  
							}
						}
					}
					
					// Atualiza Produto
					$this->updateProduto(
								$temNutriente,
								$dados['IDCONSNUT'],
								$produto['VRPESOUNID'],
								$dados['IDFATCORR'],
								$produto['VRPEFATOCORR'],
								$produto['CDPRODUTO']
						  );
									  
					// Atualiza data de retorno na Log SITPROD
					$this->atualizaSitProd($produto['NRREMESSA'],$produto['CDPRODUTO'],$produto['IDATIVO'],$dados['NRREMESSA']);			
				}
			} 
			else
			{
				throw new \Exception('Quantidade de produtos informado difere da quantidade de produtos recebidos');
			}
		}
	}
	
    private function verificaEnvioProdutos($dados) {
      if(empty($dados['PRODUTOS'])) 
          return false;
      else
          return true;
    }
    
    
  // INSERTS
  private function insertNutrProd($cdProduto,$dados){
      $insert = "INSERT INTO NUTRPROD
                  (CDPRODUTO,CDNUTRIENTE,QTNUTRPROD,DTULTATUNUPR,DSFONTNUTR,DTINCLUSAO,DTULTATU,IDATIVO, CDOPERULTATU, CDOPERINCLUSAO)
                  VALUES (:CDPRODUTO,:CDNUTRIENTE,:QTNUTRPROD,sysdate,:DSFONTNUTR,sysdate,sysdate, 'S', :CDOPERADOR, :CDOPERADOR)";
      $params = array(
          ':CDPRODUTO' => $cdProduto,
          ':CDNUTRIENTE' => $dados['CDNUTRIENTE'],
          ':QTNUTRPROD' => floatval($dados['QTNUTRPROD']),
          ':DSFONTNUTR' => $dados['DSFONTNUTR'],
          ':CDOPERADOR' => '1'
      );
     
      $stmt = $this->connection->prepare($insert);
      $stmt->execute($params);
  }

  private function insertLogProd($remessa,$cdproduto, $nrorgint) {
        $insert = "INSERT INTO SIT_LOGPROD
                    (NRREMESSA,CDPRODUTO,DTENVIO,DTINCLUSAO,DTULTATU, NRORGINT)
                    VALUES
                    (:NRREMESSA,:CDPRODUTO,SYSDATE,SYSDATE,SYSDATE, :NRORGINT)";
    
        $params = array(
            ':NRREMESSA' => $remessa,
            ':CDPRODUTO' => $cdproduto,
            ':NRORGINT'  => $nrorgint
        );
        
        $stmt = $this->connection->prepare($insert);
        $stmt->execute($params);
      
  }
  
  //Cria nova remessa
  private function criaNovaRemessa($nrRemessa, $nrorg) {
      try{
          $insert = "INSERT INTO REMPRODNUTR
                    (NRREMESSA,NRORGINT,DTENVIO,IDSITREMESSA,DTINCLUSAO,DTULTATU)
                    VALUES (:NRREMESSA,:NRORGINT,sysdate,'S',sysdate,sysdate)";
          $params = array (
              ':NRREMESSA'  => $nrRemessa,
              ':NRORGINT'   => $nrorg
          );
          
          $stmt = $this->connection->prepare($insert);
          $stmt->execute($params);
          
          return $nrRemessa;
      }catch (\Exception $e) {
          try{
             if(strpos($e->getMessage(), "PK_REMPRODNUTR") !== false){
                 $codigo = ltrim($this->retornaNrRemessa(false), '0');
                 
                 $update = "UPDATE NOVOCODIGO
                               SET NRSEQUENCIAL = :NRSEQUENCIAL,
                                   DTULTATU = SYSDATE
                             WHERE CDCONTADOR LIKE 'REMPRODNUTR'";
                 $params = array (
                      ':NRSEQUENCIAL' => str_pad(++$codigo, 20, "0", STR_PAD_LEFT) 
                  );
                 
                 $stmt = $this->connection->prepare($update);
                 $stmt->execute($params);
                 $this->criaNovaRemessa($codigo, $nrorg);
                 
             }else throw new \Exception($e->getMessage());
              
          }catch (\Exception $e) {
              throw new \Exception($e->getMessage());
          }
         
      }  
  }
  
  
  //UPDATE
  private function updateProduto($temNutriente,$idConsNut,$vrPesoUnid,$idFatCorr,$vrPeFatoCorr,$cdProduto) {
    $vrPeFatoCorr = !$vrPeFatoCorr ? 1 : $vrPeFatoCorr;
    $vrPesoUnid   = !$vrPesoUnid ? 1 : $vrPesoUnid;
    $update = "UPDATE PRODUTO
               SET IDTEMNUTRIENTE = :IDTEMNUTRIENTE,
                IDALTERANUTR = 'N',
                DTATUALNUTR = SYSDATE,
                DSOBSNUTR = NULL,
                VRPESOUNID = CASE WHEN :IDCONSNUT = 'S' THEN 
                    CASE WHEN :IDTEMNUTRIENTE = 'S' THEN $vrPesoUnid ELSE VRPESOUNID END
                ELSE VRPESOUNID END,
                VRPEFATOCORR = CASE WHEN :IDFATCORR = 'S' THEN $vrPeFatoCorr ELSE VRPEFATOCORR END,
                DTULTATU = SYSDATE
               WHERE CDPRODUTO = :CDPRODUTO";
                
    $params = array(
         ':IDTEMNUTRIENTE' => $temNutriente,
         ':IDCONSNUT' => $idConsNut,
         ':IDFATCORR' => $idFatCorr,
         ':CDPRODUTO' => $cdProduto
    );
    
    $this->connection->executeUpdate($update, $params);
  }
  
  private function atualizaSitProd($nrRemessa,$cdProduto,$temNutriente,$remessaRet = NULL) {
    
      $update = "UPDATE SIT_LOGPROD 
                 SET DTRETORNO = sysdate,
                     NRREMESSARET = :NRREMESSARET,
					 IDINFONUTRI = :IDINFONUTRI,
					 DTULTATU = SYSDATE
                 WHERE NRREMESSA = :NRREMESSA AND CDPRODUTO = :CDPRODUTO";
      
      $params = array(':NRREMESSA' => $nrRemessa,
                      ':CDPRODUTO' => $cdProduto,
                      ':NRREMESSARET' => $remessaRet,
					  ':IDINFONUTRI' => $temNutriente
                      );
      
      $stmt = $this->connection->prepare($update);
      $stmt->execute($params);
  }
  
  private function atualizaProdutoEmReceita() {
      $update = "UPDATE PRODUTO
                  SET IDTEMNUTRIENTE = 'S',
                  IDALTERANUTR = CASE WHEN EXISTS (SELECT 1 FROM NUTRPROD WHERE CDPRODUTO = PRODUTO.CDPRODUTO) AND IDTEMNUTRIENTE = 'N' THEN 'S' ELSE IDALTERANUTR END,
                  DTULTATU = SYSDATE
                  WHERE CDPRODUTO IN
                  (SELECT CDPRODUTO
                  FROM (SELECT DISTINCT CDPRODUTO
                  FROM RECEPADR
                  UNION
                  SELECT DISTINCT CDPRODUTO
                  FROM RECEFILI
                  UNION
                  SELECT DISTINCT CDPRODUTO
                  FROM RECESERV
                  UNION
                  SELECT DISTINCT CDPRODUTO
                  FROM RECETPCARD) P
                  WHERE NOT EXISTS (SELECT 1
                  FROM SIT_LOGPROD S
                  WHERE S.CDPRODUTO = P.CDPRODUTO)
                  AND NOT EXISTS (SELECT *
                                    FROM (SELECT DISTINCT CDPRODPRODUCAO 
                                            FROM PRATPADR
                                           WHERE CDPRODPRODUCAO IS NOT NULL 
                                           UNION 
                                          SELECT DISTINCT CDPRODPRODUCAO 
                                            FROM PRATFILI 
                                           WHERE CDPRODPRODUCAO IS NOT NULL 
                                           UNION 
                                          SELECT DISTINCT CDPRODPRODUCAO 
                                            FROM PRATSERV 
                                           WHERE CDPRODPRODUCAO IS NOT NULL
                                         ) REC 
                                   WHERE REC.CDPRODPRODUCAO = P.CDPRODUTO))";
        $stmt = $this->connection->prepare($update);
        $stmt->execute();          
  }
  
  private function atualizaDataEnvio($remessa) {
      // Atualiza SIT_LOGPRD
      $updateLogProd = "UPDATE SIT_LOGPROD 
                 SET DTENVIO = SYSDATE,
                     DTULTATU = SYSDATE
                 WHERE NRREMESSA = :NRREMESSA";
      
      $params = array(':NRREMESSA' => $remessa);
      
      $stmt = $this->connection->prepare($updateLogProd);
      $stmt->execute($params);
      
      // Atualiza REMPRODNUTR
      $update = "UPDATE REMPRODNUTR 
                 SET DTENVIO = SYSDATE,
                 DTULTATU = SYSDATE
                 WHERE NRREMESSA = :NRREMESSA";
      
      $params = array(':NRREMESSA' => $remessa);
      
      $stmt = $this->connection->prepare($update);
      $stmt->execute($params);  
  }
  
  //DELETE
  private function deleteNutrProd($CODPROD) {
      $sql = "DELETE FROM NUTRPROD WHERE CDPRODUTO = :CDPRODUTO";
      $params = array("CDPRODUTO" => $CODPROD);
      $stmt = $this->connection->prepare($sql);
      $stmt->execute($params);
  }
  
  
  //SELECT
  private function retornaNrRemessa($novoCodigo){
      if($novoCodigo)$resultado = $this->novoCodigo->getCodeByTableName('REMPRODNUTR',null);
      else{
          $consulta = "SELECT MAX(NRREMESSA) AS NRREMESSA FROM REMPRODNUTR";
          $resultado = $this->connection->fetchColumn($consulta);
      }
      return $resultado;
  }
  
  private function retornaDadosProduto() {
      
      $consulta = "SELECT DISTINCT
                        P.CDPRODUTO,
                        P.CDARVPROD,
                        P.NMPRODUTO,
                        P.SGUNIDADE,
                        ( P.DSOBSNUTR ) AS OBSERVACAO,
                        P.VRPESOUNID,
                        SYSDATE AS DATA,
                        NVL(IDALTERANUTR, 'N') IDALTERANUTR
                    FROM
                        PRODUTO P
                        LEFT JOIN SIT_LOGPROD S ON ( S.CDPRODUTO = P.CDPRODUTO )
                                                   AND ( S.DTRETORNO IS NULL )
                        LEFT JOIN (
                            SELECT
                                CDPRODUTO,
                                MAX(1) AS EXIST_PROD
                            FROM
                                NUTRPROD 
                            GROUP BY
                                CDPRODUTO
                        ) N ON N.CDPRODUTO = P.CDPRODUTO
                    WHERE
                        ( ROWNUM <= 1000 )
                        AND   ( P.IDTEMNUTRIENTE = 'S' )
                        AND   ( P.IDPRODATIVPR = 'S' )
                        AND   NOT EXISTS (SELECT * 
                                            FROM (SELECT DISTINCT CDPRODPRODUCAO 
                                                    FROM PRATPADR
                                                   WHERE CDPRODPRODUCAO IS NOT NULL 
                                                   UNION 
                                                  SELECT DISTINCT CDPRODPRODUCAO 
                                                    FROM PRATFILI 
                                                   WHERE CDPRODPRODUCAO IS NOT NULL 
                                                   UNION 
                                                  SELECT DISTINCT CDPRODPRODUCAO 
                                                    FROM PRATSERV 
                                                   WHERE CDPRODPRODUCAO IS NOT NULL
                                                 ) REC 
                                           WHERE REC.CDPRODPRODUCAO = P.CDPRODUTO)                     
                        AND   ( S.CDPRODUTO IS NULL )
                        AND   ( ( N.CDPRODUTO IS NULL )  OR ( NVL(IDALTERANUTR, 'N') = 'S' ))
                    ORDER BY
                        P.CDPRODUTO";
      
      $resultado = $this->connection->fetchAll($consulta);
      
      return $resultado;
  }
  
  private function retornaDadosJaEnviados($remessa, $nrorgint){
	$consulta = "SELECT S.CDPRODUTO,
                        P.CDARVPROD,
                        P.NMPRODUTO,
                        P.SGUNIDADE,
                        ( P.DSOBSNUTR ) AS OBSERVACAO,
                        P.VRPESOUNID,
                        S.DTENVIO AS DATA,
                        NVL(P.IDALTERANUTR, 'N') IDALTERANUTR
                   FROM SIT_LOGPROD S
				  INNER JOIN PRODUTO P ON P.CDPRODUTO = S.CDPRODUTO
                  WHERE S.NRREMESSA = :NRREMESSA
				    AND (S.NRORGINT = :NRORGINT OR S.NRORGINT IS NULL)
                  ORDER BY S.CDPRODUTO";
            
      $params = array("NRREMESSA" => $remessa, "NRORGINT" => $nrorgint);
      $resultado = $this->connection->fetchAll($consulta, $params);
      
	  return $resultado;
  }
  private function verificaGrupNutr($cdgrupnutr,$nmgrupnutr) {
      if (!empty($cdgrupnutr)) {
          $consulta = "SELECT CDGRUPNUTR FROM GRUPNUTR WHERE CDGRUPNUTR = :CDGRUPNUTR";
		  
		  $params = array("CDGRUPNUTR" => $cdgrupnutr);
		  $resultado = $this->connection->fetchAll($consulta, $params);
		  
		  if(empty($resultado)) {
			  
			$insert = "INSERT INTO GRUPNUTR 
				(CDGRUPNUTR,NMGRUPNUTR, DTULTATUGRNU, DTINCLUSAO, DTULTATU, IDATIVO, CDOPERULTATU, CDOPERINCLUSAO) 
				VALUES(:CDGRUPNUTR,:NMGRUPNUTR,sysdate,sysdate,sysdate, 'S', :CDOPERADOR, :CDOPERADOR)";
		
			$params = array(
				':CDGRUPNUTR' => $cdgrupnutr,
				':NMGRUPNUTR' => $nmgrupnutr,
				':CDOPERADOR' => '1'
			);
			
			$stmt = $this->connection->prepare($insert);
			$stmt->execute($params);
			
		  } else {
			  
			  $update = "UPDATE GRUPNUTR
					  SET  DTULTATUGRNU = SYSDATE, 
							NMGRUPNUTR = :NMGRUPNUTR,
							DTULTATU = SYSDATE
					  WHERE CDGRUPNUTR = :CDGRUPNUTR";
		  
			  $params = array(
				  ':NMGRUPNUTR' => $nmgrupnutr,
				  ':CDGRUPNUTR' => $cdgrupnutr
			  );
			  
			  $stmt = $this->connection->prepare($update);
			  $stmt->execute($params);
		  }
	  }
  }
  
  private function verificaUnidade($sgunidade, $dsunidade) {
	  if (!empty($sgunidade)){
		  $consulta = "SELECT SGUNIDADE FROM UNIDMEDIDA WHERE SGUNIDADE = :SGUNIDADE";
		  
		  $params = array("SGUNIDADE" => $sgunidade);
		  $resultado = $this->connection->fetchAll($consulta, $params);
		  
		  if(empty($resultado)) {
			  $insert = "INSERT INTO UNIDMEDIDA 
					  (SGUNIDADE,DSUNIDADE, DTINCLUSAO, DTULTATU, IDATIVO, CDOPERINCLUSAO, CDOPERULTATU) 
					  VALUES(:SGUNIDADE,:DSUNIDADE,sysdate,sysdate,'S', :CDOPERADOR, :CDOPERADOR)";
			  $params = array(
				  ':SGUNIDADE' => $sgunidade,
				  ':DSUNIDADE' => $dsunidade == null ? $sgunidade : $dsunidade,
				  ':CDOPERADOR' => '1'
			  );
			  
			  $stmt = $this->connection->prepare($insert);
			  $stmt->execute($params);
		  }else {
			  $update = "UPDATE UNIDMEDIDA
					  SET  DTULTATUUNID = SYSDATE,
					  DTULTATU = SYSDATE
					  WHERE SGUNIDADE = :SGUNIDADE";
		  
			  $params = array(
				  ':SGUNIDADE' => $sgunidade
			  );
			  
			  $stmt = $this->connection->prepare($update);
			  $stmt->execute($params);
		  }
	  }
  }
  
  private function verificaNutriente($param) {
      
      $consulta = "SELECT CDNUTRIENTE FROM NUTRIENTE WHERE CDNUTRIENTE = :CDNUTRIENTE";
      
      $params = array("CDNUTRIENTE" => $param['CDNUTRIENTE']);
      $resultado = $this->connection->fetchAll($consulta, $params);
      
      if(empty($resultado)) {
          if(!empty($param['CDNUTRIENTE'])){
              $insert = "INSERT INTO NUTRIENTE
                      (CDNUTRIENTE,NMNUTRIENTE,NMREDUZIDO,IDTIPONUTR,SGUNIDADE,CDGRUPNUTR,DTULTATUNUTR,IDCALORIAS,IDNUTRRELEV,IDKCAL,NRORDEMNUTR,DTINCLUSAO,DTULTATU, IDATIVO, CDOPERINCLUSAO, CDOPERULTATU)
                     VALUES (:CDNUTRIENTE,:NMNUTRIENTE,:NMREDUZIDO,:IDTIPONUTR,:SGUNIDADE,:CDGRUPNUTR,sysdate,:IDCALORIAS,:IDNUTRRELEV,:IDKCAL,:NRORDEMNUTR,sysdate,sysdate, 'S', :CDOPERADOR, :CDOPERADOR)";
          
              if(isset($param['IDKCAL']))
                  $idkcal = $param['IDKCAL'];
              else
                  $idkcal = 'N';
              
              $params = array(
                  ':CDNUTRIENTE' =>  $param['CDNUTRIENTE'],
                  ':NMNUTRIENTE' =>  $param['NMNUTRIENTE'],
                  ':NMREDUZIDO' =>  $param['NMREDUZIDO'],
                  ':IDTIPONUTR' =>  $param['IDTIPONUTR'],
                  ':SGUNIDADE' =>  $param['SGUNIDADE'],
                  ':CDGRUPNUTR' =>  $param['CDGRUPNUTR'],
                  ':IDCALORIAS' =>  $param['IDCALORIAS'],
                  ':IDNUTRRELEV' =>  $param['IDNUTRRELEV'],
                  ':IDKCAL' =>  $idkcal, 
                  ':NRORDEMNUTR' =>  $param['NRORDEMNUTR'],
                  ':CDOPERADOR' => '1'
              );
              
              $stmt = $this->connection->prepare($insert);
              $stmt->execute($params);
          }
      } else {
          
          $update = "UPDATE NUTRIENTE
                  SET NMNUTRIENTE = :NMNUTRIENTE,
                  NMREDUZIDO = :NMREDUZIDO, 
                  IDTIPONUTR = :IDTIPONUTR, 
                  SGUNIDADE = :SGUNIDADE, 
                  CDGRUPNUTR = :CDGRUPNUTR, 
                  DTULTATUNUTR = SYSDATE,
                  IDCALORIAS = :IDCALORIAS,
                  IDNUTRRELEV = :IDNUTRRELEV,
                  VRFATORNUTR = :VRFATORNUTR,
                  DTULTATU = SYSDATE
              WHERE CDNUTRIENTE = :CDNUTRIENTE";
              
          $params = array(        
              ':NMNUTRIENTE' => $param['NMNUTRIENTE'],
              ':NMREDUZIDO' => $param['NMREDUZIDO'],
              ':IDTIPONUTR' => $param['IDTIPONUTR'],
              ':SGUNIDADE' => $param['SGUNIDADE'],
              ':CDGRUPNUTR' => $param['CDGRUPNUTR'],
              ':IDCALORIAS' => $param['IDCALORIAS'],
              ':IDNUTRRELEV' => $param['IDNUTRRELEV'],
              ':VRFATORNUTR' => $param['VRFATORNUTR'],
              ':CDNUTRIENTE' => $param['CDNUTRIENTE']
          );
          
          $stmt = $this->connection->prepare($update);
          $stmt->execute($params);
          
      }
  }
	
  private function validaOrganizacao($nrorg){
      $consulta = "SELECT NRORGINTEGRA FROM PARAMGERAL";
      $nrOrgCliente = $this->connection->fetchColumn($consulta);
      if ($nrOrgCliente != $nrorg) {
	      throw new \Exception("Organização ($nrorg - $nrOrgCliente) incompatível com os dados da Teknisa Software.");
	  }
      return true;
  }
  
  private function configNovaVersao(){
    //Apaga info do endereço da versão antiga
    $query = "UPDATE PARAMGERAL SET DSENDEINTEGRA = NULL";
    $stmt = $this->connection->prepare($query);
    $stmt->execute();

  }
}
